@@ -1,5 +1,6 @@
 # Author: Christian Brodbeck <christianbrodbeck@nyu.edu>
 """Pre-processing operations based on NDVars"""
+import warnings
 from copy import deepcopy
 import fnmatch
 from itertools import chain
@@ -14,7 +15,7 @@ from scipy import signal
 from .. import load
 from .._data_obj import NDVar, Sensor
 from .._exceptions import DefinitionError
-from .._io.fiff import KIT_NEIGHBORS
+from .._io.fiff import KIT_NEIGHBORS, find_mne_channel_types
 from ..mne_fixes._version import MNE_VERSION, V0_19
 from .._ndvar import filter_data
 from .._text import enumeration
@@ -150,6 +151,33 @@ class RawSource(RawPipe):
     See Also
     --------
     MneExperiment.raw
+
+    Examples
+    --------
+    If you would load the EEG data like this::
+
+        form pathlib import Path
+
+        subject = '0001'
+        raw = mne.io.read_raw_brainvision(
+            f'/data/eeg/{subject}/raw/task_{subject}.vhdr',
+            eog=['EOG horizontal left', 'EOG horizontal right', 'EOG vertical below', 'EOG vertical above'],
+        )
+        raw.rename_channels({'FZ': 'Fz', 'OZ': 'Oz', 'CZ': 'Cz', 'Ref right mastoid': 'A1'})
+        raw.set_montage('standard_1020')
+
+    Then the equivalent :class:`RawSource` is::
+
+        raw = {
+            'raw': RawSource(
+                'raw/{recording}_{subject}.vhdr',
+                reader=mne.io.read_raw_brainvision,
+                rename_channels={'FZ': 'Fz', 'OZ': 'Oz', 'CZ': 'Cz', 'Ref right mastoid': 'A1'},
+                eog=['EOG horizontal left', 'EOG horizontal right', 'EOG vertical below', 'EOG vertical above'],
+                montage='standard_1020'),
+            ...
+        }
+
     """
     _dig_sessions: dict = None  # {subject: {for_recording: use_recording}}
     bads_path: str = None  # set on linking
@@ -275,9 +303,13 @@ class RawSource(RawPipe):
         with open(path, 'w') as fid:
             fid.write(text)
 
-    def make_bad_channels_auto(self, subject, recording, flat=1e-14, redo=False):
+    def make_bad_channels_auto(self, subject, recording, flat=None, redo=False):
         raw = self.load(subject, recording, add_bads=False)
         bad_chs = raw.info['bads']
+        if flat is None:
+            # flat EEG channel is probably the reference
+            if find_mne_channel_types(raw.info) != ['eeg']:
+                flat = 1e-14
         if flat:
             raw = load.fiff.raw_ndvar(raw)
             bad_chs.extend(raw.sensor.names[raw.std('time') < flat])
@@ -563,7 +595,9 @@ class RawICA(CachedRawPipe):
         path = self._ica_path(subject, visit)
         if not exists(path):
             raise FileMissing(f"ICA file {basename(path)} does not exist for raw={self.name!r}. Run e.make_ica_selection() to create it.")
-        return mne.preprocessing.read_ica(path)
+        with warnings.catch_warnings():
+            warnings.filterwarnings('ignore', 'Version 0.23 introduced max_iter', DeprecationWarning)
+            return mne.preprocessing.read_ica(path)
 
     @staticmethod
     def _check_ica_channels(
@@ -799,7 +833,6 @@ class RawReReference(CachedRawPipe):
     --------
     MneExperiment.raw
     """
-
     def __init__(self, source, reference='average', add=None, drop=None, cache=False):
         CachedRawPipe.__init__(self, source, cache)
         if not isinstance(reference, str):
@@ -822,6 +855,12 @@ class RawReReference(CachedRawPipe):
         raw = self.source.load(subject, recording, preload=True)
         if self.add:
             raw = mne.add_reference_channels(raw, self.add, copy=False)
+            # apply new channel position
+            pipe = self.source
+            while not isinstance(pipe, RawSource):
+                pipe = pipe.source
+            if pipe.montage:
+                raw.set_montage(pipe.montage)
         raw.set_eeg_reference(self.reference)
         if self.drop:
             raw = raw.drop_channels(self.drop)
