@@ -12,7 +12,7 @@ from itertools import groupby, repeat, zip_longest
 from math import floor
 from numbers import Real
 import operator
-from typing import Any, Callable, Sequence, Union
+from typing import Any, Callable, Literal, Sequence, Union
 
 import mne
 from numba import njit, prange
@@ -86,7 +86,8 @@ def concatenate(
     name
         Name the NDVar holding the result.
     tmin : float | 'first'
-        Time axis start, only applies when ``dim == 'time'``; default is 0.
+        Time axis start, only applies when concatenating along time dimension
+        (``dim='time'``); default is 0.
         Set ``tmin='first'`` to use ``tmin`` of ``ndvars[0]``.
     info
         Info for the returned ``ndvar``.
@@ -142,16 +143,14 @@ def concatenate(
             ndvars = [complete_source_space(v, to=out_dim, mask=False) for v in ndvars]
             x = reduce(np.add, [v.get_data(dim_names) for v in ndvars])
         else:
-            x = np.concatenate([v.get_data(dim_names) for v in ndvars], axis)
-            if isinstance(dim_obj, UTS):
+            out_dim = dim_obj._concatenate([v.get_dim(dim) for v in ndvars])
+            if isinstance(out_dim, UTS):
                 if isinstance(tmin, str):
-                    if tmin == 'first':
-                        tmin = ndvar.time.tmin
-                    else:
+                    if tmin != 'first':
                         raise ValueError(f"tmin={tmin!r}")
-                out_dim = UTS(tmin, ndvar.time.tstep, x.shape[axis])
-            else:
-                out_dim = dim_obj._concatenate(v.get_dim(dim) for v in ndvars)
+                else:
+                    out_dim = set_tmin(out_dim, tmin)
+            x = np.concatenate([v.get_data(dim_names) for v in ndvars], axis)
         dims = ndvar.get_dims(dim_names)
         dims = (*dims[:axis], out_dim, *dims[axis+1:])
     return NDVar(x, dims, name or ndvar.name, info)
@@ -167,12 +166,21 @@ def _concatenate_values(
             return concatenate(values, dim)
         elif not all((v == values[0]).all() for v in values[1:]):
             raise ValueError(f'Inconsistent values for {key}: {values}')
+    if isinstance(values[0], Dimension):
+        if values[0].name == dim:
+            return values[0]._concatenate(values)
+        elif not all(dim_i == values[0] for dim_i in values[1:]):
+            raise ValueError(f'Inconsistent values for {key}: {values}')
+        else:
+            return values[:1]
     elif isinstance(values[0], (tuple, list)):
         if isinstance(values[0][0], NDVar) and values[0][0].has_dim(dim):
-            items = [concatenate(items, dim) for items in zip(*values)]
+            items = [concatenate(items, dim) for items in zip_longest(*values)]
             if isinstance(values[0], tuple):
                 items = tuple(items)
             return items
+        elif isinstance(values[0][0], Dimension):
+            return [_concatenate_values(items, dim, f'{key}[{i}') for i, items in enumerate(zip_longest(*values))]
         for values_i in zip_longest(*values):
             value_i_0 = values_i[0]
             if isinstance(value_i_0, (NDVar, np.ndarray)):
@@ -360,7 +368,10 @@ def correlation_coefficient(x, y, dim=None, name=None):
     if np.any(isnan):
         np.place(out, isnan, 0)
     dims = x.get_dims(shared_dims + x_only) + y.get_dims(y_only)
-    return NDVar(out, dims, name or x.name, x.info)
+    if name is None:
+        name = x.name
+    info = _info.for_stat_map('r', old=x.info)
+    return NDVar(out, dims, name, info)
 
 
 def cross_correlation(in1, in2, name=None):
@@ -548,7 +559,8 @@ def filter_data(
     Returns
     -------
     filtered_ndvar
-        NDVar with same dimensions as ``ndvar`` and filtered data.
+        NDVar with same dimensions as ``ndvar`` and filtered data. The output
+        type is always floating point regardless of the type of ``ndvar``.
     """
     axis = ndvar.get_axis('time')
     if axis == ndvar.ndim:
@@ -558,6 +570,9 @@ def filter_data(
         data = ndvar.x.swapaxes(axis, -1)
     sfreq = 1. / ndvar.time.tstep
 
+    if data.dtype.kind != 'f':
+        data = data.astype(float)
+
     filter_kwargs.setdefault('copy', True)
     x = mne.filter.filter_data(data, sfreq, low, high, **filter_kwargs)
 
@@ -1072,25 +1087,32 @@ def rename_dim(ndvar, old_name, new_name):
     return NDVar(ndvar.x, dims, ndvar.name, ndvar.info)
 
 
-def resample(ndvar, sfreq, npad='auto', window=None, pad='edge', name=None):
+def resample(
+        ndvar: NDVar,
+        sfreq: float,
+        npad: Union[int, Literal['auto']] = 'auto',
+        window: Union[str, tuple] = None,
+        pad: str = 'edge',
+        name: str = None,
+) -> NDVar:
     """Resample an NDVar along the time dimension
 
     Parameters
     ----------
-    ndvar : NDVar
-        Input data.
-    sfreq : scalar
+    ndvar
+        Input data, with :class:`UTS` time dimension.
+    sfreq
         New sampling frequency.
-    npad : int | 'auto'
+    npad
         Number of samples for padding at the beginning and end (default is
         determined automatically).
-    window : str | tuple
+    window
         Window applied to the signal in the fourier domain (default is no
         window; see :func:`scipy.signal.resample`).
-    pad : str
+    pad
         Padding method (default ``'edge'``; see :func:`numpy.pad` ``mode``
         parameter).
-    name : str
+    name
         Name for the new NDVar (default is ``ndvar.name``).
 
     Notes
@@ -1105,20 +1127,22 @@ def resample(ndvar, sfreq, npad='auto', window=None, pad='edge', name=None):
     if name is None:
         name = ndvar.name
     axis = ndvar.get_axis('time')
+    time: UTS = ndvar.get_dim('time')
     new_tstep = 1. / sfreq
     if npad:
-        old_sfreq = 1.0 / ndvar.time.tstep
-        x = mne.filter.resample(ndvar.x, sfreq, old_sfreq, npad, axis, window, pad=pad)
+        old_sfreq = 1.0 / time.tstep
+        x = ndvar.x if ndvar.x.dtype.kind == 'f' else ndvar.x.astype(float)
+        x = mne.filter.resample(x, sfreq, old_sfreq, npad, axis, window, pad=pad)
         new_num = x.shape[axis]
         if isinstance(ndvar.x, np.ma.masked_array):
             mask = mne.filter.resample(ndvar.x.mask.astype(float), sfreq, old_sfreq, npad, axis, window, pad=pad)
             x = np.ma.masked_array(x, mask > 0.5)
     else:
-        new_num = int(floor((ndvar.time.tstop - ndvar.time.tmin) / new_tstep))
+        new_num = int(floor((time.tstop - time.tmin) / new_tstep))
         # crop input data
         new_duration = new_tstep * new_num
-        old_num = int(round(new_duration / ndvar.time.tstep))
-        if old_num == ndvar.time.nsamples:
+        old_num = int(round(new_duration / time.tstep))
+        if old_num == time.nsamples:
             idx = None
         else:
             idx = (slice(None),) * axis + (slice(None, old_num),)
@@ -1129,7 +1153,7 @@ def resample(ndvar, sfreq, npad='auto', window=None, pad='edge', name=None):
             mask = ndvar.x.mask if idx is None else ndvar.x.mask[idx]
             mask = signal.resample(mask.astype(float), new_num, axis=axis, window=window)
             x = np.ma.masked_array(x, mask > 0.5)
-    time_dim = UTS(ndvar.time.tmin, new_tstep, new_num)
+    time_dim = UTS(time.tmin, new_tstep, new_num)
     dims = (*ndvar.dims[:axis], time_dim, *ndvar.dims[axis + 1:])
     return NDVar(x, dims, name, ndvar.info)
 
@@ -1269,6 +1293,7 @@ def set_parc(
         parc: Union[str, Factor],
         dim: str = 'source',
         mask: bool = False,
+        name: str = None,
 ) -> Union[NDVar, SourceSpace]:
     """Change the parcellation of an :class:`NDVar` or :class:`SourceSpace` dimension
 
@@ -1285,6 +1310,8 @@ def set_parc(
         Name of the dimension to operate on (usually ``'source'``, the default).
     mask
         Remove ``unknown-*`` vertices.
+    name
+        Name for the new NDVar.
 
     Returns
     -------
@@ -1295,10 +1322,12 @@ def set_parc(
     if isinstance(data, SourceSpaceBase):
         source = out = data._copy(parc=parc)
     elif isinstance(data, NDVar):
+        if name is None:
+            name = data.name
         axis = data.get_axis(dim)
         source = set_parc(data.dims[axis], parc, mask=False)
         dims = (*data.dims[:axis], source, *data.dims[axis + 1:])
-        out = NDVar(data.x, dims, data.name, data.info)
+        out = NDVar(data.x, dims, name, data.info)
     else:
         raise TypeError(data)
     if mask:
@@ -1312,34 +1341,49 @@ def set_parc(
     return out
 
 
-def set_tmin(ndvar, tmin=0.):
+def set_tmin(
+        data: Union[NDVar, UTS],
+        tmin: float = 0.,
+        name: str = None,
+) -> Union[NDVar, UTS]:
     """Shift the time axis of an :class:`NDVar` relative to its data
 
     Parameters
     ----------
-    tmin : scalar
+    data
+        :class:`NDVar` or :class:`UTS` dimension on which to set ``tmin``.
+    tmin
         New ``tmin`` value (default 0).
+    name
+        Name for the new NDVar.
 
     Returns
     -------
-    out_ndvar : NDVar
-        Shallow copy of ``ndvar`` with updated time axis.
+    out
+        Shallow copy of ``data`` with updated time axis.
 
     See Also
     --------
     set_time : Pad/crop the :class:`NDVar`
     """
-    axis = ndvar.get_axis('time')
-    old = ndvar.dims[axis]
-    dims = list(ndvar.dims)
+    if isinstance(data, UTS):
+        return UTS(tmin, data.tstep, data.nsamples)
+    elif not isinstance(data, NDVar):
+        raise TypeError(f'{data=}')
+    if name is None:
+        name = data.name
+    axis = data.get_axis('time')
+    old: UTS = data.dims[axis]
+    dims = list(data.dims)
     dims[axis] = UTS(tmin, old.tstep, old.nsamples)
-    return NDVar(ndvar.x, dims, ndvar.name, ndvar.info)
+    return NDVar(data.x, dims, name, data.info)
 
 
 def set_time(
         ndvar: NDVar,
         time: Union[NDVar, UTS],
         mode: str = 'constant',
+        name: str = None,
         **kwargs,
 ):
     """Crop and/or pad an :class:`NDVar` to match the time axis ``time``
@@ -1352,6 +1396,8 @@ def set_time(
         New time axis, or :class:`NDVar` with time axis to match.
     mode : str
         How to pad ``ndvar``, see :func:`numpy.pad`.
+    name
+        Name for the new NDVar.
     **
         See :func:`numpy.pad`.
 
@@ -1361,6 +1407,8 @@ def set_time(
     """
     if isinstance(time, NDVar):
         time = time.get_dim('time')
+    if name is None:
+        name = ndvar.name
     axis = ndvar.get_axis('time')
     ndvar_time = ndvar.get_dim('time')
     if ndvar_time.tstep != time.tstep:
@@ -1380,4 +1428,4 @@ def set_time(
         pad_width = [*repeat(no_pad, axis), pad, *repeat(no_pad, ndvar.ndim-axis-1)]
         x = np.pad(x, pad_width, mode, **kwargs)
     dims = [*ndvar.dims[:axis], time, *ndvar.dims[axis+1:]]
-    return NDVar(x, dims, ndvar.name, ndvar.info)
+    return NDVar(x, dims, name, ndvar.info)
