@@ -2,7 +2,7 @@
 """Statistical tests for univariate variables"""
 from functools import partial
 import itertools
-from typing import Literal, Sequence, Union
+from typing import Dict, Literal, Sequence, Union
 
 import numpy as np
 import scipy.stats
@@ -40,7 +40,7 @@ def get_levels(
         else:
             return DEFAULT_LEVELS
     elif trend:
-        raise TypeError("trend=%r only valid when levels=True" % (trend,))
+        raise TypeError(f"{trend=} only valid when levels=True")
     return levels
 
 
@@ -103,6 +103,10 @@ class Correlation:
     def _asfmtext(self, **_):
         return fmtxt.FMText([fmtxt.eq('r', self.r, self.df), ', ', fmtxt.peq(self.p)])
 
+    @property
+    def stars(self):
+        return fmtxt.Stars.from_p(self.p)
+
 
 def lilliefors(data, formatted=False, **kwargs):
     """Lilliefors' test for normal distribution
@@ -550,6 +554,10 @@ class TTest:
         self.p = stats.ttest_p(self.t, self.df, tail)
         self.tail = tail
 
+    @property
+    def stars(self):
+        return fmtxt.Stars.from_p(self.p)
+
     def _asfmtext(
             self,
             rasterize: bool = None,
@@ -804,7 +812,7 @@ class MannWhitneyU:
         elif tail == -1:
             alternative = 'less'
         else:
-            raise ValueError(f"tail={tail!r}")
+            raise ValueError(f"{tail=}")
         self.u, self.p = scipy.stats.mannwhitneyu(y1.x, y0.x, continuity, alternative)
         self._y = dataobj_repr(y1)
         self._x = x_name
@@ -825,6 +833,10 @@ class MannWhitneyU:
     def _asfmtext(self, **_):
         return fmtxt.FMText([fmtxt.eq('U', self.u), ', ', fmtxt.peq(self.p)])
 
+    @property
+    def stars(self):
+        return fmtxt.Stars.from_p(self.p)
+
 
 class TTestRelated(TTest):
     """Related-measures t-test
@@ -1045,6 +1057,10 @@ class WilcoxonSignedRank:
     def _asfmtext(self, **_):
         return fmtxt.FMText([fmtxt.eq('W', self.w), ', ', fmtxt.peq(self.p)])
 
+    @property
+    def stars(self):
+        return fmtxt.Stars.from_p(self.p)
+
 
 def pairwise(
         y: VarArg,
@@ -1053,10 +1069,11 @@ def pairwise(
         sub: IndexArg = None,
         ds: Dataset = None,
         par: bool = True,
-        corr: Union[None, str] = 'Hochberg',
+        corr: MCCArg = 'Hochberg',
         trend: Union[bool, str] = False,
         title: str = '{desc}',
         mirror: bool = False,
+        labels: Dict[CellArg, str] = None,
 ):
     """Pairwise comparison table
 
@@ -1084,6 +1101,8 @@ def pairwise(
         Title for the table.
     mirror
         Redundant table including all row/column combinations.
+    labels
+        Alternative labels for ``x`` as ``{cell: label}`` dictionary.
 
     Returns
     -------
@@ -1099,20 +1118,21 @@ def pairwise(
     statistic = test['statistic']
     _K = test[statistic]
     _P = test['p']
-    if corr:
-        _Pc = mcp_adjust(_P, corr)
+    _Pc = mcp_adjust(_P, corr)
     _df = test['df']
     symbols = test['symbols']
+    cellnames = ct.cellnames()
+    if labels:
+        cellnames = [labels.get(name, name) for name in cellnames]
 
     # create TABLE
     table = fmtxt.Table('l' + 'l' * (k - 1 + mirror))
-    title_desc = "Pairwise {0}".format(test['test'])
+    title_desc = f"Pairwise {test['test']}"
     table.title(title.format(desc=title_desc))
     table.caption(test['caption'])
 
     # headings
     table.cell()
-    cellnames = ct.cellnames()
     for name in cellnames[1 - mirror:]:
         table.cell(name)
     table.midrule()
@@ -1125,29 +1145,22 @@ def pairwise(
 
     n_stars = 3 + bool(trend)
     for row in range(0, k - 1 + mirror):
-        for subrow in range(subrows):  # contains t/p
-            # names column
-            if subrow == 0:
-                table.cell(cellnames[row])
+        table.cell(cellnames[row])
+        for col in range(1 - mirror, k):
+            if row == col:
+                table.cell()
+            elif mirror or col > row:
+                index = indexes[(row, col)]
+                content = [
+                    fmtxt.eq(statistic, _K[index], _df[index], stars=symbols[index], of=n_stars),
+                    fmtxt.linebreak,
+                    fmtxt.peq(_P[index]),
+                ]
+                if corr:
+                    content.extend([fmtxt.linebreak, fmtxt.peq(_Pc[index], 'c')])
+                table.cell(content)
             else:
                 table.cell()
-            # rows
-            for col in range(1 - mirror, k):
-                if row == col:
-                    table.cell()
-                elif col > row:
-                    index = indexes[(row, col)]
-                    if subrow == 0:
-                        table.cell(fmtxt.eq(statistic, _K[index], _df[index], stars=symbols[index], of=n_stars))
-                    elif subrow == 1:
-                        table.cell(fmtxt.peq(_P[index]))
-                    elif subrow == 2:
-                        table.cell(fmtxt.peq(_Pc[index], 'c'))
-                elif mirror and corr and subrow == 0:
-                    index = indexes[(col, row)]
-                    table.cell(fmtxt.P(_Pc[index]))
-                else:
-                    table.cell()
     return table
 
 
@@ -1232,7 +1245,12 @@ def _pairwise(
     return out
 
 
-def pairwise_correlations(xs, sub=None, ds=None, labels={}):
+def pairwise_correlations(
+        xs: Sequence[Union[str, Var, NDVar]],
+        sub: IndexArg = None,
+        ds: Dataset = None,
+        labels: Dict[CellArg, str] = None,
+):
     """Pairwise correlation table
 
     Parameters
@@ -1245,7 +1263,7 @@ def pairwise_correlations(xs, sub=None, ds=None, labels={}):
         If a Dataset is given, all data-objects can be specified as names of
         Dataset variables.
     labels : {str: str} dict
-        Labels for ``xs`` in the table (mapping ``x.name`` to string labels).
+        Alternative labels for ``xs`` as ``{x.name: label}`` dictionary.
 
     Returns
     -------
@@ -1253,25 +1271,24 @@ def pairwise_correlations(xs, sub=None, ds=None, labels={}):
         Table with pairwise correlations.
     """
     sub = assub(sub, ds)
-    x_rows = [asnumeric(x, sub, ds) for x in xs]
-    n_vars = len(x_rows)
+    xs_ = [asnumeric(x, sub, ds) for x in xs]
+    n_vars = len(xs_)
+    x_labels = [nice_label(x, labels) for x in xs_]
 
     table = fmtxt.Table('l' + 'c' * n_vars)
     # header
-    table.cell()
-    for i in range(1, n_vars + 1):
-        table.cell(i)
+    table.cells('', *x_labels)
     table.midrule()
     # body
-    for i_row, x_row in enumerate(x_rows):
-        label = nice_label(x_row, labels)
-        table.cell("%i  %s" % (i_row + 1, label))
-        for x_col in x_rows:
+    for i_row, x_row in enumerate(xs_):
+        table.cell(x_labels[i_row])
+        for x_col in xs_:
             if x_col is x_row:
                 table.cell()
             else:
                 corr = Correlation(x_row, x_col)
-                table.cell(fmtxt.stat(corr.r, drop0=True))
+                cell = fmtxt.FMText([fmtxt.eq('r', corr.r, corr.df), fmtxt.Stars.from_p(corr.p), fmtxt.linebreak, fmtxt.peq(corr.p)])
+                table.cell(cell)
         table.endline()
     return table
 
