@@ -18,7 +18,7 @@ from ._data_obj import (
     cellname, combine, isuv)
 
 
-def difference(y, x, c1, c0, match, sub=None, ds=None, by=None):
+def difference(y, x, c1, c0, match, sub=None, ds=None):
     """Subtract data in one cell from another
 
     Parameters
@@ -73,57 +73,20 @@ def difference(y, x, c1, c0, match, sub=None, ds=None, by=None):
     sub = assub(sub, ds)
     x = ascategorial(x, sub, ds)
     out = Dataset()
-    if by is None:
-        ct = Celltable(y, x, match, sub, ds=ds)
-        if not ct.all_within:
-            raise ValueError("Design is not fully balanced")
-        groups = ct.groups[c1]
-        if isinstance(groups, Interaction):
-            for x in groups.base:
-                out.add(x)
-        else:
-            out.add(groups)
-        yname = y if isinstance(y, str) else ct.y.name
-        out[yname] = ct.data[c1] - ct.data[c0]
-        # Transfer other variables in ds that are compatible with the rm-structure
-        if ds is not None:
-            out.update(ct._align_ds(ds, True, out.keys(), isuv))
+    ct = Celltable(y, x, match, sub, ds=ds)
+    if not ct.all_within:
+        raise ValueError("Design is not fully balanced")
+    groups = ct.groups[c1]
+    if isinstance(groups, Interaction):
+        for x in groups.base:
+            out.add(x)
     else:
-        warn("The by parameter is deprecated; use match instead", DeprecationWarning)
-        by = ascategorial(by, sub, ds)
-        ct = Celltable(y, x % by, match, sub, ds=ds)
-        if not ct.all_within:
-            raise ValueError("Design is not fully balanced")
-
-        yname = y if isinstance(y, str) else ct.y.name
-        if isinstance(c1, str):
-            c1 = (c1,)
-        if isinstance(c0, str):
-            c0 = (c0,)
-        dss = []
-        ds_keep = None
-        for cell in by.cells:
-            if isinstance(cell, str):
-                cell = (cell,)
-            cell_ds = Dataset()
-            cell_ds.add(ct.groups[c1 + cell])
-            cell_ds[yname] = ct.data[c1 + cell] - ct.data[c0 + cell]
-            if isinstance(by, Factor):
-                cell_ds[by.name, :] = cell[0]
-            else:
-                for b, c in zip(by.base, cell):
-                    cell_ds[b.name, :] = c
-            # Transfer other variables in ds that are compatible with the rm-structure
-            if ds_keep is None:
-                if ds is None:
-                    ds_keep = False
-                else:
-                    ds_keep = ct._align_ds(ds, True, cell_ds.keys(), isuv)
-            if ds_keep:
-                cell_ds.update(ds_keep)
-            dss.append(cell_ds)
-        out = combine(dss)
-
+        out.add(groups)
+    yname = y if isinstance(y, str) else ct.y.name
+    out[yname] = ct.data[c1] - ct.data[c0]
+    # Transfer other variables in ds that are compatible with the rm-structure
+    if ds is not None:
+        out.update(ct._align_ds(ds, True, out.keys(), isuv))
     return out
 
 
@@ -499,12 +462,12 @@ def cast_to_ndvar(
 
     # determine NDVar dimension
     if isinstance(dim_values, Factor):
-        unique_dim_vales = dim_values.cells
-        dim = Categorial(dim or dim_values.name, unique_dim_vales)
+        unique_dim_values = dim_values.cells
+        dim = Categorial(dim or dim_values.name, unique_dim_values)
     else:
-        unique_dim_vales = np.unique(dim_values.x)
+        unique_dim_values = np.unique(dim_values.x)
         if dim == 'uts':
-            diff = np.diff(unique_dim_vales)
+            diff = np.diff(unique_dim_values)
             unique_diff = np.unique(diff)
             if len(unique_diff) > 1:
                 if np.diff(unique_diff).max() > 1e-15:
@@ -512,22 +475,26 @@ def cast_to_ndvar(
                 tstep = round(unique_diff.mean(), 17)
             else:
                 tstep = unique_diff[0]
-            dim = UTS(unique_dim_vales[0], tstep, len(unique_dim_vales), unit)
+            dim = UTS(unique_dim_values[0], tstep, len(unique_dim_values), unit)
         else:
-            dim = Scalar(dim or dim_values.name, unique_dim_vales)
+            dim = Scalar(dim or dim_values.name, unique_dim_values)
 
     # find NDVar data
     n_samples = len(dim)
     n_cases = len(match.cells)
     case_indexes = [match == case for case in match.cells]
-    samples_indexes = [dim_values == v for v in unique_dim_vales]
+    samples_indexes = [dim_values == v for v in unique_dim_values]
     xs = [np.empty((n_cases, n_samples)) for _ in data_vars]
     index = None
     for i, case_index in enumerate(case_indexes):
-        for j, sample_index in enumerate(samples_indexes):
-            index = np.logical_and(case_index, sample_index, out=index)
-            for x, data_var in zip(xs, data_vars):
-                x[i, j] = data_var.x[index]
+        try:
+            for j, sample_index in enumerate(samples_indexes):
+                index = np.logical_and(case_index, sample_index, out=index)
+                for x, data_var in zip(xs, data_vars):
+                    x[i, j] = data_var.x[index]
+        except ValueError:
+            if not np.any(index):
+                raise ValueError(f"Case {match.cells[i]!r} is missing some values")
 
     # package output dataset
     if ds is None:
@@ -547,7 +514,7 @@ def stats(
         match: CategorialArg = None,
         sub: IndexArg = None,
         fmt: str = '%.4g',
-        funcs: Sequence[Callable] = (np.mean,),
+        funcs: Sequence[Union[str, Callable]] = ('mean',),
         ds: Dataset = None,
         title: fmtxt.FMTextArg = None,
         caption: fmtxt.FMTextArg = None,
@@ -571,7 +538,8 @@ def stats(
         How to format values.
     funcs
         A list of statistics functions to show (all functions must take an
-        array argument and return a scalar).
+        array argument and return a scalar; strings are interpreted as
+        :mod:`numpy` functions).
     ds
         If a Dataset is provided, ``y``, ``row``, and ``col`` can be strings
         specifying members.
@@ -597,7 +565,7 @@ def stats(
     a0   0.1668    -0.3646
     a1   -0.4897   0.8746
 
-    >>> table.stats('Y', 'A', ds=ds, funcs=[np.mean, np.std])
+    >>> table.stats('Y', 'A', ds=ds, funcs=['mean', 'std'])
     Condition   Mean     Std
     --------------------------
     a0          0.6691   1.37
@@ -610,6 +578,10 @@ def stats(
     if match is not None:
         match = ascategorial(match, sub, ds)
 
+    if isinstance(funcs, str):
+        funcs = [funcs]
+    funcs = [getattr(np, f) if isinstance(f, str) else f for f in funcs]
+
     if col is None:
         ct = Celltable(y, row, match=match)
 
