@@ -245,7 +245,7 @@ class SharedToolsMenu:
         menu.AppendSubMenu(blmenu, "Baseline")
 
     def OnFindNoisyEpochs(self, event):
-        dlg = FindNoisyEpochsDialog(self)
+        dlg = FindNoisyEpochsDialog(self, unit=self.doc.epochs_ndvar.info.get('unit', '?'))
         rcode = dlg.ShowModal()
         dlg.Destroy()
         if rcode != wx.ID_OK:
@@ -555,6 +555,13 @@ class Frame(SharedToolsMenu, FileFrame):
         top_component = epoch_comp_power.argmax()
         self.GoToComponentEpoch(component=top_component)
 
+    def FindTopEpoch(self, i_comp: int):
+        source = self.doc.sources.sub(component=i_comp)
+        y = source - source.mean()
+        y **= 2
+        ss = y.sum('time')  # ndvar has epoch as index
+        self.GoToComponentEpoch(epoch=ss.argmax())
+
     def GoToComponentEpoch(self, component: int = None, epoch: int = None):
         if not self.source_frame:
             self.ShowSources(0)
@@ -597,6 +604,9 @@ class Frame(SharedToolsMenu, FileFrame):
     def OnFindTopComponent(self, event):
         self.FindTopComponent(event.EventObject.i_epoch)
 
+    def OnFindTopEpoch(self, event):
+        self.FindTopEpoch(event.EventObject.i_comp)
+
     def OnPanelResize(self, event):
         w, h = event.GetSize()
         n_h = w // self.ax_size
@@ -615,10 +625,9 @@ class Frame(SharedToolsMenu, FileFrame):
         except RuntimeError:
             return  # can be called after the window closes (Windows)
         if event.inaxes:
-            sb.SetStatusText("#%i of %i ICA Components" %
-                             (event.inaxes.i, len(self.doc.components)))
+            sb.SetStatusText(f"#{event.inaxes.i} of len(self.doc.components) ICA Components")
         else:
-            sb.SetStatusText("%i ICA Components" % len(self.doc.components))
+            sb.SetStatusText(f"{len(self.doc.components)} ICA Components")
 
     def OnRankEpochs(self, event):
         i_comp = event.EventObject.i_comp
@@ -642,6 +651,8 @@ class Frame(SharedToolsMenu, FileFrame):
     def _context_menu(self, i_comp: int = None, i_epoch: int = None):
         menu = ContextMenu(i_comp, i_epoch)
         if i_comp is not None:
+            item = menu.Append(wx.ID_ANY, "Top Epoch")
+            self.Bind(wx.EVT_MENU, self.OnFindTopEpoch, item)
             item = menu.Append(wx.ID_ANY, "Rank Epochs")
             self.Bind(wx.EVT_MENU, self.OnRankEpochs, item)
             item = menu.Append(wx.ID_ANY, "Plot Topomap")
@@ -917,7 +928,7 @@ class SourceFrame(SharedToolsMenu, FileFrameChild):
         if event.inaxes:
             if event.inaxes.i_comp is None:
                 i_in_axes = ceil(event.ydata / self.y_scale + 0.5)
-                if i_in_axes == 0 and self.show_range:
+                if i_in_axes == 1 and self.show_range:
                     return
                 i_comp = int(self.i_first + self.n_comp + self.show_range - i_in_axes)
                 if i_comp < self.n_comp_in_ica:
@@ -1215,7 +1226,7 @@ class SourceFrame(SharedToolsMenu, FileFrameChild):
 class FindNoisyEpochsDialog(EelbrainDialog):
     _default_threshold = 1.5e-12
 
-    def __init__(self, parent, *args, **kwargs):
+    def __init__(self, parent, *args, unit, **kwargs):
         super(FindNoisyEpochsDialog, self).__init__(parent, wx.ID_ANY, "Find Bad Epochs", *args, **kwargs)
         config = parent.config
         threshold = config.ReadFloat("FindNoisyEpochsDialog/threshold", self._default_threshold)
@@ -1224,7 +1235,7 @@ class FindNoisyEpochsDialog(EelbrainDialog):
         sizer = wx.BoxSizer(wx.VERTICAL)
 
         # Threshold
-        sizer.Add(wx.StaticText(self, label="Threshold for bad epochs [T]:"))
+        sizer.Add(wx.StaticText(self, label=f"Threshold for bad epochs [{unit}]:"))
         validator = REValidator(POS_FLOAT_PATTERN, "Invalid entry: {value}. Please specify a number > 0.", False)
         self.threshold = ctrl = wx.TextCtrl(self, value=str(threshold), validator=validator)
         ctrl.SetHelpText("Find epochs in which the signal exceeds this value at any sensor")
