@@ -17,7 +17,7 @@ import numpy as np
 from .._data_obj import NDVar, SourceSpace, asndvar
 from .._exceptions import KeysMissing
 from .._text import ms
-from .._utils import LazyProperty
+from .._utils import IS_OSX, LazyProperty
 from ..fmtxt import Image
 from ..mne_fixes import reset_logger
 from ._base import CONFIG, TimeSlicer, AxisScale, do_autorun, find_fig_cmaps, find_fig_vlims, fix_vlim_for_cmap, use_inline_backend
@@ -275,10 +275,10 @@ class Brain(TimeSlicer, surfer.Brain):
             raise ValueError("Trying to add NDVar without rh data to plot of rh")
         return source
 
-    def add_label(self, label, color=None, alpha=1, scalar_thresh=None,
-                  borders=False, hemi=None, subdir=None):
+    def add_label(self, label, color=None, alpha=1, scalar_thresh=None, borders=False, hemi=None, subdir=None, lighting=False, **kwargs):
         surfer.Brain.add_label(self, label, color, alpha, scalar_thresh,
                                borders, hemi, subdir)
+        self.labels_dict[label.name][0].actor.property.lighting = lighting
         if color is None:
             color = getattr(label, 'color', None) or "crimson"
         name = label if isinstance(label, str) else label.name
@@ -404,10 +404,7 @@ class Brain(TimeSlicer, surfer.Brain):
 
         # make sure time axis is compatible with existing data
         if time_dim is not None:
-            if self._time_dim is None:
-                self._init_time_dim(time_dim)
-            elif time_dim != self._time_dim:
-                raise ValueError(f"The brain already displays an NDVar with incompatible time dimension (current: {self._time_dim};  new: {time_dim})")
+            self._init_time_dim(time_dim)
 
         # find colormap parameters
         meas = ndvar.info.get('meas')
@@ -1012,14 +1009,19 @@ class Brain(TimeSlicer, surfer.Brain):
             else:
                 scale = 75
 
+        i = 0
         for figs in self._figures:
             for fig in figs:
                 if forward is not None or up is not None:
                     mlab.view(focalpoint=(0, forward or 0, up or 0), figure=fig)
                 if scale is not None:
-                    fig.scene.camera.parallel_scale = scale
+                    if IS_OSX and self._frame and i == 0:
+                        fig.scene.camera.parallel_scale = 2 * scale
+                    else:
+                        fig.scene.camera.parallel_scale = scale
                 fig.scene.camera.parallel_projection = True
                 fig.render()
+                i += 1
 
     def set_size(self, width, height):
         """Set image size in pixels"""
